==> msci.sdk-1.5.0-py3-none-any.whl: msci/sdk/calc/portfolio/mos/v1_4/batch_jobs.py <==
import datetime
import json
import logging
import time
import uuid
import pandas as pd

from .bulk_result import BulkResult
from ...utils.request_utility import get, post, delete
from ...utils.utility import bulk_request_body


class BatchJob:

    def __init__(self, base_url, headers, ssl_verify):

        self.logger = logging.getLogger(__name__)
        self.base_url = base_url
        self.headers = headers
        self.ssl_verify = ssl_verify

    def execute_jobs(self, profile_list: list) -> pd.DataFrame:
        """
        Executes list of profiles in batched mechanism with each batch allowing maximum 1000 profiles.
        Batch Id is created corresponding to entire batch processed and returned as output along with the job details.

        Args:
            profile_list (list): List of profiles to be included in the bulk execution request.

        Returns:
            Dataframe consisting of jobId, statusCode, errorText, batchId and profileName.
        """
        batch_size = 1000
        resp_list = []
        batch_res = pd.DataFrame()
        batch_id = uuid.uuid4()
        self.logger.info(f"Creating bulk job upload request with batch_id: {batch_id}")

        for i in range(0, len(profile_list), batch_size):
            pf_lst = profile_list[i:i + batch_size]
            payload = [prof.body for prof in pf_lst]
            profile_names = [prof['requestInfo']['name'] for prof in payload]
            resp = post(url=self.base_url + f"bulkJobs?batchId={batch_id}", headers=self.headers,
                        data=payload)
            response_json = json.loads(resp.text)
            self.logger.debug(response_json)
            response_df = pd.json_normalize(response_json)
            response_df['batchId'] = batch_id
            response_df['profileName'] = profile_names
            resp_list.append(response_df)

        if resp_list:
            batch_res = pd.concat(resp_list, ignore_index=True)
        return batch_res

    def get_jobs_status(self, batch_id: str):
        """
        Use to check if all jobs in a batch are completed (failed counts as complete).
        You will need the batch ID to call this endpoint.

        Args:
            batch_id (str): Batch ID of the batch for which the job statuses are to be retrieved.

        Returns:
            Response dict
        """
        resp = get(url=self.base_url + f"batchJobs/{batch_id}/complete", headers=self.headers)
        return json.loads(resp.text)

    def get_batch_info(self, batch_id: str):
        """
        Read info about batch. Contains a list of all associated jobIds and profileNames. This does not include profiles that were rejected from the batch.

        Args:
            batch_id (str): Batch ID of the batch for which the job IDs are to be retrieved.

        Returns:
            Response dict
        """
        resp = get(url=self.base_url + f"batch/info/{batch_id}", headers=self.headers)
        return json.loads(resp.text)

    def _get_batch_response(self, job_id_list: list, as_of_date: str = None, snapshot_types: list = None):
        """
        Get compound results for many jobs in a single call for a specified list of jobs and a specified result types.

        Args:
            job_id_list (list): List of job IDs for which results are to be fetched.
            as_of_dates (list): (optional) List of as_of_dates.
            snapshot_types (list): (optional) List of snapshot_types for the portfolios.
                            Required if portfolios or tax lots need to be included in the results.

        Returns:
            Response dict
        """
        keys = bulk_request_body(job_id_list, as_of_date, snapshot_types)
        body = {
            "keys": keys,
            "includePortfolios": False,
            "includeTaxLots": False,
            "includeTradeSuggestions": True if as_of_date else False,
            "includeValuations": True,
            "includeMetrics": True,
            "includeTriggerDates": True,
            "includeBenchmarkIds": True,
            "includeJobWarnings": True,
            "includeProfile": True
        }
        if as_of_date and snapshot_types:
            body['includePortfolios'] = True
            body['includeTaxLots'] = True

        resp = post(url=self.base_url + f"jobs/bulkResults", headers=self.headers, data=body)
        return resp.json()

    def cancel_jobs(self, job_ids: list):
        """
        Use to cancel/delete a set of jobs. You will need the IDs of the jobs to use this endpoint.

        Args:
            job_ids (list): List of job IDs.

        Returns:
            Response dict
        """
        self.logger.debug(f"Canceling bulk job upload request with job ids: {job_ids}")
        resp = post(url=self.base_url + f"bulkJobs/cancel", headers=self.headers, data=job_ids)
        return json.loads(resp.text)

    def delete_job(self, job_id: str):
        """
        Use to delete/cancel a job. You will need the ID of the job to use this endpoint.

        Args:
            job_id (str): Job ID of the active job that has to be cancelled.

        Returns:
            None
        """
        self.logger.debug(f"Deleting job with job id: {job_id}")
        resp = delete(url=self.base_url + f"jobs/{job_id}", headers=self.headers)
        if resp.status_code == 200:
            self.logger.info(f"Job deleted with job id: {job_id}.")
        else:
            self.logger.info(f"Job delete failed for job id: {job_id}.")

    def wait(self, batch_id: str, wait_time: int = 5):
        start_time = datetime.datetime.now()
        batch_status = self.get_jobs_status(batch_id)
        while not batch_status["complete"]:
            self.logger.info(f"Batch status is not yet completed. Retrying after {wait_time} secs.")
            self.logger.info(f"{batch_status}")
            time.sleep(wait_time)
            batch_status = self.get_jobs_status(batch_id)
        td = (datetime.datetime.now() - start_time).total_seconds()
        self.logger.info(f"Batch status completed in {td} seconds.")
        return batch_status

    def bulk_results(self, job_id_list: list, as_of_date: str = None,
                     snapshot_types: list = ['REBALANCED']):
        """
        Get compound results for many jobs in a single call for a specified list of jobs and a specified result type.

        Args:
            job_id_list (list): List of job IDs for which results need to be fetched.
            as_of_date (list): (optional) List of as_of_dates.
            snapshot_types (list): (optional) List of snapshot_types for the portfolios. Required if portfolios or tax lots need to be included in the results.

        Returns:
            Response dict
        """

        job_id_list = [job for job in job_id_list if job]
        if not job_id_list:
            self.logger.info("No successful jobs found to retrieve the bulk results")
            return
        bulk_response = self._get_batch_response(job_id_list, as_of_date=as_of_date, snapshot_types=snapshot_types)
        return BulkResult(bulk_response)

==> msci.sdk-1.6.0-py3-none-any.whl: msci/sdk/calc/portfolio/mos/v1_4/batch_jobs.py <==
import datetime
import json
import logging
import time
import uuid
import pandas as pd

from .bulk_result import BulkResult
from ...utils.request_utility import get, post, delete
from ...utils.utility import bulk_request_body


class BatchJob:

    def __init__(self, base_url, headers, ssl_verify):

        self.logger = logging.getLogger(__name__)
        self.base_url = base_url
        self.headers = headers
        self.ssl_verify = ssl_verify

    def execute_jobs(self, profile_list: list) -> pd.DataFrame:
        """
        Executes list of profiles in batched mechanism with each batch allowing maximum 1000 profiles.
        Batch Id is created corresponding to entire batch processed and returned as output along with the job details.

        Args:
            profile_list (list): List of profiles to be included in the bulk execution request.

        Returns:
            Dataframe consisting of jobId, statusCode, errorText, batchId and profileName.
        """
        batch_size = 1000
        resp_list = []
        batch_res = pd.DataFrame()
        batch_id = uuid.uuid4()
        self.logger.info(f"Creating bulk job upload request with batch_id: {batch_id}")

        for i in range(0, len(profile_list), batch_size):
            pf_lst = profile_list[i:i + batch_size]
            payload = [prof.body for prof in pf_lst]
            profile_names = [prof['requestInfo']['name'] for prof in payload]
            resp = post(url=self.base_url + f"bulkJobs?batchId={batch_id}", headers=self.headers,
                        data=payload, ssl_verify=self.ssl_verify)
            response_json = json.loads(resp.text)
            self.logger.debug(response_json)
            response_df = pd.json_normalize(response_json)
            response_df['batchId'] = batch_id
            response_df['profileName'] = profile_names
            resp_list.append(response_df)

        if resp_list:
            batch_res = pd.concat(resp_list, ignore_index=True)
        return batch_res

    def get_jobs_status(self, batch_id: str):
        """
        Use to check if all jobs in a batch are completed (failed counts as complete).
        You will need the batch ID to call this endpoint.

        Args:
            batch_id (str): Batch ID of the batch for which the job statuses are to be retrieved.

        Returns:
            Response dict
        """
        resp = get(url=self.base_url + f"batchJobs/{batch_id}/complete", headers=self.headers, ssl_verify=self.ssl_verify)
        return json.loads(resp.text)

    def get_batch_info(self, batch_id: str):
        """
        Read info about batch. Contains a list of all associated jobIds and profileNames. This does not include profiles that were rejected from the batch.

        Args:
            batch_id (str): Batch ID of the batch for which the job IDs are to be retrieved.

        Returns:
            Response dict
        """
        resp = get(url=self.base_url + f"batch/info/{batch_id}", headers=self.headers, ssl_verify=self.ssl_verify)
        return json.loads(resp.text)

    def _get_batch_response(self, job_id_list: list, as_of_date: str = None, snapshot_types: list = None):
        """
        Get compound results for many jobs in a single call for a specified list of jobs and a specified result types.

        Args:
            job_id_list (list): List of job IDs for which results are to be fetched.
            as_of_dates (list): (optional) List of as_of_dates.
            snapshot_types (list): (optional) List of snapshot_types for the portfolios.
                            Required if portfolios or tax lots need to be included in the results.

        Returns:
            Response dict
        """
        keys = bulk_request_body(job_id_list, as_of_date, snapshot_types)
        body = {
            "keys": keys,
            "includePortfolios": False,
            "includeTaxLots": False,
            "includeTradeSuggestions": True if as_of_date else False,
            "includeValuations": True,
            "includeMetrics": True,
            "includeTriggerDates": True,
            "includeBenchmarkIds": True,
            "includeJobWarnings": True,
            "includeProfile": True
        }
        if as_of_date and snapshot_types:
            body['includePortfolios'] = True
            body['includeTaxLots'] = True

        resp = post(url=self.base_url + f"jobs/bulkResults", headers=self.headers, data=body, ssl_verify=self.ssl_verify)
        return resp.json()

    def cancel_jobs(self, job_ids: list):
        """
        Use to cancel/delete a set of jobs. You will need the IDs of the jobs to use this endpoint.

        Args:
            job_ids (list): List of job IDs.

        Returns:
            Response dict
        """
        self.logger.debug(f"Canceling bulk job upload request with job ids: {job_ids}")
        resp = post(url=self.base_url + f"bulkJobs/cancel", headers=self.headers, data=job_ids, ssl_verify=self.ssl_verify)
        return json.loads(resp.text)

    def delete_job(self, job_id: str):
        """
        Use to delete/cancel a job. You will need the ID of the job to use this endpoint.

        Args:
            job_id (str): Job ID of the active job that has to be cancelled.

        Returns:
            None
        """
        self.logger.debug(f"Deleting job with job id: {job_id}")
        resp = delete(url=self.base_url + f"jobs/{job_id}", headers=self.headers, ssl_verify=self.ssl_verify)
        if resp.status_code == 200:
            self.logger.info(f"Job deleted with job id: {job_id}.")
        else:
            self.logger.info(f"Job delete failed for job id: {job_id}.")

    def wait(self, batch_id: str, wait_time: int = 5):
        start_time = datetime.datetime.now()
        batch_status = self.get_jobs_status(batch_id)
        while not batch_status["complete"]:
            self.logger.info(f"Batch status is not yet completed. Retrying after {wait_time} secs.")
            self.logger.info(f"{batch_status}")
            time.sleep(wait_time)
            batch_status = self.get_jobs_status(batch_id)
        td = (datetime.datetime.now() - start_time).total_seconds()
        self.logger.info(f"Batch status completed in {td} seconds.")
        return batch_status

    def bulk_results(self, job_id_list: list, as_of_date: str = None,
                     snapshot_types: list = ['REBALANCED']):
        """
        Get compound results for many jobs in a single call for a specified list of jobs and a specified result type.

        Args:
            job_id_list (list): List of job IDs for which results need to be fetched.
            as_of_date (list): (optional) List of as_of_dates.
            snapshot_types (list): (optional) List of snapshot_types for the portfolios. Required if portfolios or tax lots need to be included in the results.

        Returns:
            Response dict
        """

        job_id_list = [job for job in job_id_list if job]
        if not job_id_list:
            self.logger.info("No successful jobs found to retrieve the bulk results")
            return
        bulk_response = self._get_batch_response(job_id_list, as_of_date=as_of_date, snapshot_types=snapshot_types)
        return BulkResult(bulk_response)
